# This is a demo to use DPHEM which is compatible with sklearn.mixture.GaussianMixture 
# ---
# If you use this code, please cite our paper:
#   Lei Yu, Tianyu Yang, and Antoni B. Chan. "Density Preserving Hierarchical EM Algorithm: Simplifying 
#   Gaussian Mixture Models for Approximate Inference." TPAMI 2019
# ---
# Copyright (c) 2019-04-30
# Lei Yu, Antoni B. Chan
# Department of Computer Science
# City University of Hong Kong 
#
# v0.1

import numpy as np
import matplotlib.pyplot as plt
from sklearn.mixture import GaussianMixture
from sklearn.datasets.samples_generator import make_blobs
from sklearn.datasets import load_wine
from pylib_dphem import *
from gmm_dphem_learn import gmm_dphem_learn
from random import gauss
from random import seed

demo_1d = True

# An example of basegmm fitted from randomly generated data or loaded toy datasets
# 1D example
if demo_1d:
    seed(1)
    X = []
    for i in range(200):
        if i < 50:
            X.append(gauss(0,0.5))
        elif i < 100:
            X.append(gauss(5,1))
        elif i < 150:
            X.append(gauss(10,0.5))
        else:
            X.append(gauss(15,1))

    X = np.asarray(X).reshape(-1,1)
    basegmm = GaussianMixture(n_components=10,covariance_type='full',random_state=1).fit(X)
  
else:
    # 2D example
    X, y_true = make_blobs(n_samples=1000, centers=10,
                           cluster_std=1, random_state=0)
    basegmm = GaussianMixture(n_components=20,covariance_type='full',random_state=1).fit(X)


# number of components in the target reduced gmm
tagk = 4
# pimode: 
#   - preserve : DPHEM
#   - original : HEM
# initmode:
#   - random: initialize the reducedgmm with randomly selected basegmm components
#   - wtkmeans: initialize the reducedgmm with weighted kmeans clustering of basegmm components
opt = {'pimode': 'preserve', 
       'initmode': 'random'}

reducedgmm,post,info,diffel = gmm_dphem_learn(basegmm,tagk,opt)


plt.figure(0)
if demo_1d:
    gmm_plot1d(basegmm, 'b')
else:
    plot_gmm(basegmm,X)

plt.figure(1)
if demo_1d:
    gmm_plot1d(reducedgmm, 'b')
else:
    plot_gmm(reducedgmm,X)

plt.show()










